@@ -106,10 +106,6 @@ list_free (GList *objects)
 static void
 g_gdu_volume_monitor_dispose (GObject *object)
 {
-  GGduVolumeMonitor *monitor;
-
-  monitor = G_GDU_VOLUME_MONITOR (object);
-
   the_volume_monitor = NULL;
 
   if (G_OBJECT_CLASS (g_gdu_volume_monitor_parent_class)->dispose)
@@ -638,6 +634,10 @@ _g_unix_mount_point_guess_should_display (GUnixMountPoint *mount_point)
   if (g_unix_is_mount_path_system_internal (mount_path))
     return FALSE;
 
+  /* Hide mounts within a dot path, suppose it was a purpose to hide this mount */
+  if (g_strstr_len (mount_path, -1, "/.") != NULL)
+    return FALSE;
+
   /* Only display things in /media (which are generally user mountable)
      and home dir (fuse stuff) */
   if (g_str_has_prefix (mount_path, "/media/"))
@@ -849,9 +849,7 @@ should_drive_be_ignored (GduPool *pool, GduDrive *d, GList *fstab_mount_points)
   device = gdu_presentable_get_device (GDU_PRESENTABLE (d));
 
   /* If there is no GduDevice for a drive, then ignore it unless
-   * we know how to start it. Right now this is only relevant
-   * for GduLinuxMdDrive but we can add other stuff to libgdu
-   * in the future and things will work here
+   * we know how to start it (e.g. RAID arrays, Volume Groups)
    */
   if (device == NULL)
     {
@@ -949,10 +947,11 @@ should_drive_be_ignored (GduPool *pool, GduDrive *d, GList *fstab_mount_points)
             ignored = TRUE;
         }
 
-      /* special case for audio discs: don't ignore the drive since we'll create
-       * a cdda:// mount for the drive
+      /* special case for audio and blank discs: don't ignore the drive since we'll create
+       * a cdda:// or burn:// mount for the drive
        */
-      if (gdu_device_is_optical_disc (device) && gdu_device_optical_disc_get_num_audio_tracks (device) > 0)
+      if (gdu_device_is_optical_disc (device) && (gdu_device_optical_disc_get_num_audio_tracks (device) > 0 ||
+                                                  gdu_device_optical_disc_get_is_blank (device)))
         {
           ignored = FALSE;
         }
@@ -1061,16 +1060,37 @@ find_disc_volume_for_device_file (GGduVolumeMonitor *monitor,
                                   const gchar       *device_file)
 {
   GList *l;
+  GGduVolume *ret;
+  struct stat stat_buf;
+
+  ret = NULL;
+
+  if (stat (device_file, &stat_buf) == 0)
+    {
+      for (l = monitor->disc_volumes; l != NULL; l = l->next)
+        {
+          GGduVolume *volume = G_GDU_VOLUME (l->data);
+          if (g_gdu_volume_has_dev (volume, stat_buf.st_rdev))
+            {
+              ret = volume;
+              goto out;
+            }
+        }
+    }
 
   for (l = monitor->disc_volumes; l != NULL; l = l->next)
     {
       GGduVolume *volume = G_GDU_VOLUME (l->data);
 
       if (g_gdu_volume_has_device_file (volume, device_file))
-        return volume;
+        {
+          ret = volume;
+          goto out;
+        }
     }
 
-  return NULL;
+ out:
+  return ret;
 }
 
 static GGduVolume *
@@ -1079,8 +1099,23 @@ find_volume_for_device_file (GGduVolumeMonitor *monitor,
 {
   GList *l;
   GGduVolume *ret;
+  struct stat stat_buf;
 
   ret = NULL;
+
+  if (stat (device_file, &stat_buf) == 0)
+    {
+      for (l = monitor->volumes; l != NULL; l = l->next)
+        {
+          GGduVolume *volume = G_GDU_VOLUME (l->data);
+          if (g_gdu_volume_has_dev (volume, stat_buf.st_rdev))
+            {
+              ret = volume;
+              goto out;
+            }
+        }
+    }
+
   for (l = monitor->volumes; l != NULL; l = l->next)
     {
       GGduVolume *volume = G_GDU_VOLUME (l->data);
@@ -1108,17 +1143,41 @@ find_volume_for_device_file (GGduVolumeMonitor *monitor,
   return ret;
 }
 
+static GGduVolume *
+find_volume_for_presentable (GGduVolumeMonitor *monitor,
+                             GduPresentable    *presentable)
+{
+  GList *l;
+
+  for (l = monitor->volumes; l != NULL; l = l->next)
+    {
+      GGduVolume *volume = G_GDU_VOLUME (l->data);
+
+      if (g_gdu_volume_has_presentable (volume, presentable))
+        return volume;
+    }
+
+  return NULL;
+}
+
 static GGduDrive *
 find_drive_by_device_file (GGduVolumeMonitor *monitor,
                            const gchar       *device_file)
 {
   GList *l;
+  struct stat stat_buf;
+
+  if (stat (device_file, &stat_buf) != 0)
+    {
+      g_warning ("%s:%s: Error statting %s: %m", G_STRLOC, G_STRFUNC, device_file);
+      return NULL;
+    }
 
   for (l = monitor->drives; l != NULL; l = l->next)
     {
       GGduDrive *drive = G_GDU_DRIVE (l->data);
 
-      if (g_gdu_drive_has_device_file (drive, device_file))
+      if (g_gdu_drive_has_dev (drive, stat_buf.st_rdev))
         return drive;
     }
 
@@ -1267,21 +1326,14 @@ update_volumes (GGduVolumeMonitor *monitor,
   for (l = removed; l != NULL; l = l->next)
     {
       GduPresentable *p = GDU_PRESENTABLE (l->data);
-      GduDevice *d;
 
-      d = gdu_presentable_get_device (p);
-
-      if (d != NULL)
+      volume = find_volume_for_presentable (monitor, p);
+      if (volume != NULL)
         {
-          volume = find_volume_for_device_file (monitor, gdu_device_get_device_file (d));
-          if (volume != NULL)
-            {
-              /*g_debug ("removing volume %s", gdu_device_get_device_file (d));*/
-              g_gdu_volume_removed (volume);
-              monitor->volumes = g_list_remove (monitor->volumes, volume);
-              *removed_volumes = g_list_prepend (*removed_volumes, volume);
-            }
-          g_object_unref (d);
+          /*g_debug ("removing volume %s", gdu_device_get_device_file (d));*/
+          g_gdu_volume_removed (volume);
+          monitor->volumes = g_list_remove (monitor->volumes, volume);
+          *removed_volumes = g_list_prepend (*removed_volumes, volume);
         }
     }
 
@@ -1298,29 +1350,14 @@ update_volumes (GGduVolumeMonitor *monitor,
 
       if (volume == NULL)
         {
-          GduPresentable *toplevel_presentable;
+          GduDrive *gdu_drive;
 
-          toplevel_presentable = gdu_presentable_get_toplevel (p);
-          if (toplevel_presentable != NULL)
+          drive = NULL;
+          gdu_drive = gdu_volume_get_drive (GDU_VOLUME (p));
+          if (gdu_drive != NULL)
             {
-              GduDevice *toplevel_device;
-
-              drive = NULL;
-              toplevel_device = gdu_presentable_get_device (toplevel_presentable);
-              if (toplevel_device != NULL)
-                {
-                  drive = find_drive_by_device_file (monitor, gdu_device_get_device_file (toplevel_device));
-                  /*g_debug ("adding volume %s (drive %s)",
-                           gdu_device_get_device_file (d),
-                           gdu_device_get_device_file (toplevel_device));*/
-                  g_object_unref (toplevel_device);
-                }
-              g_object_unref (toplevel_presentable);
-            }
-          else
-            {
-              drive = NULL;
-              /*g_debug ("adding volume %s (no drive)", gdu_device_get_device_file (d));*/
+              drive = find_drive_by_presentable (monitor, GDU_PRESENTABLE (gdu_drive));
+              g_object_unref (gdu_drive);
             }
 
           volume = g_gdu_volume_new (G_VOLUME_MONITOR (monitor),
@@ -1390,7 +1427,7 @@ update_fstab_volumes (GGduVolumeMonitor *monitor,
           GduDevice *device;
 
           /* doesn't exist */
-          if (realpath (device_file, resolved_path) != 0)
+          if (realpath (device_file, resolved_path) == NULL)
             continue;
 
           /* is handled by DKD */
@@ -1512,7 +1549,7 @@ update_mounts (GGduVolumeMonitor *monitor,
       if (volume == NULL)
         volume = find_volume_for_mount_path (monitor, mount_path);
 
-      /*g_debug ("adding mount %s (vol %p)", g_unix_mount_get_device_path (mount_entry), volume);*/
+      /*g_debug ("adding mount %s (vol %p) (device %s, mount_path %s)", g_unix_mount_get_device_path (mount_entry), volume, device_file, mount_path);*/
       mount = g_gdu_mount_new (G_VOLUME_MONITOR (monitor), mount_entry, volume);
       if (mount)
         {
@@ -1540,7 +1577,7 @@ update_discs (GGduVolumeMonitor *monitor,
   GList *new_discs;
   GList *removed, *added;
   GList *l, *ll;
-  GGduDrive *drive;
+  GGduDrive *drive = NULL;
   GGduVolume *volume;
   GGduMount *mount;
 
@@ -1640,24 +1677,36 @@ update_discs (GGduVolumeMonitor *monitor,
 
       if (volume == NULL)
         {
-          GduPresentable *toplevel_presentable;
+          GduPresentable *toplevel_drive;
 
-          toplevel_presentable = gdu_presentable_get_toplevel (p);
-          if (toplevel_presentable != NULL)
+          toplevel_drive = gdu_presentable_get_enclosing_presentable (p);
+          /* handle logical partitions enclosed by an extented partition */
+          if (GDU_IS_VOLUME (toplevel_drive))
             {
-              GduDevice *toplevel_device;
+              GduPresentable *temp;
+              temp = toplevel_drive;
+              toplevel_drive = gdu_presentable_get_enclosing_presentable (toplevel_drive);
+              g_object_unref (temp);
+            }
 
-              drive = NULL;
-              toplevel_device = gdu_presentable_get_device (toplevel_presentable);
-              if (toplevel_device != NULL)
+          if (toplevel_drive != NULL)
+            {
+              if (GDU_IS_DRIVE (toplevel_drive))
                 {
-                  drive = find_drive_by_device_file (monitor, gdu_device_get_device_file (toplevel_device));
-                  /*g_debug ("adding volume %s (drive %s)",
-                           gdu_device_get_device_file (d),
-                           gdu_device_get_device_file (toplevel_device));*/
-                  g_object_unref (toplevel_device);
+                  GduDevice *toplevel_drive_device;
+
+                  drive = NULL;
+                  toplevel_drive_device = gdu_presentable_get_device (toplevel_drive);
+                  if (toplevel_drive_device != NULL)
+                    {
+                      drive = find_drive_by_device_file (monitor, gdu_device_get_device_file (toplevel_drive_device));
+                      /*g_debug ("adding volume %s (drive %s)",
+                        gdu_device_get_device_file (d),
+                        gdu_device_get_device_file (toplevel_device));*/
+                      g_object_unref (toplevel_drive_device);
+                    }
                 }
-              g_object_unref (toplevel_presentable);
+              g_object_unref (toplevel_drive);
             }
           else
             {
